SystemOrganization addCategory: #Transcript!

Object subclass: #ThreadSafeTranscript
	instanceVariableNames: 'stream accessSemaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transcript'!

!ThreadSafeTranscript commentStamp: 'stephane.ducasse 4/1/2009 21:25' prior: 0!
I'm an output device.

Ultimately I can replace TranscripterStream since I'm thread safe and TranscripterStream.

ThreadSafeTranscript can be installed as the default transcript using
ThreadSafeTranscript installThreadSafeAsTranscript

It can be installed as another Transcript accessible using STranscript
ThreadSafeTranscript installThreadSafeAsSTranscript
!

!ThreadSafeTranscript class methodsFor: 'examples' stamp: 'lr 3/14/2010 21:13'!
examples
	"self examples"

	| tt |
	Smalltalk globals at: #STranscript ifAbsent: [ self installThreadSafeAsSTranscript ].
	tt := Smalltalk globals at: #STranscript.
	tt open.
	tt
		nextPutAll: 'Pharo';
		flush;
		cr;
		tab.
	tt
		show: ' is cool';
		cr.
	tt reset.
	tt clear.
	tt
		nextPutAll: 'Pharo';
		flush;
		cr;
		tab.
	tt
		show: ' is really cool';
		cr! !

!ThreadSafeTranscript class methodsFor: 'examples' stamp: 'lr 3/14/2010 21:13'!
examplesConcurrent
	"self examplesConcurrent"

	| tt |
	Smalltalk globals at: #STranscript ifAbsent: [ self installThreadSafeAsSTranscript ].
	tt := Smalltalk globals at: #STranscript.
	tt open.
	[ 
	1 to: 10 do: [ :i | 
		tt
			nextPutAll: i printString;
			nextPutAll: '*'.
		Processor yield ].
	tt flush ] fork.
	[ 
	100 to: 110 do: [ :i | 
		tt
			nextPutAll: i printString;
			nextPutAll: '-'.
		Processor yield ].
	tt flush ] fork! !

!ThreadSafeTranscript class methodsFor: 'examples' stamp: 'lr 3/14/2010 21:13'!
examplesForegroundUpdate
	"self examplesForegroundUpdate"

	| tt length |
	Smalltalk globals at: #STranscript ifAbsent: [ self installThreadSafeAsSTranscript ].
	tt := Smalltalk globals at: #STranscript.
	tt open.
	length := 20.
	tt
		cr;
		show: 'STARTING----->'.	"Foreground updates"
	1000 to: 1000 + length do: [ :i | 
		tt show: '---' , i printString , '---'.
		(Delay forSeconds: 1) wait ]! !

!ThreadSafeTranscript class methodsFor: 'examples' stamp: 'lr 3/14/2010 21:13'!
examplesHighlyConcurrent
	"self examplesHighlyConcurrent"

	| tt length |
	Smalltalk globals at: #STranscript ifAbsent: [ self installThreadSafeAsSTranscript ].
	tt := Smalltalk globals at: #STranscript.
	tt open.
	length := 20.
	tt
		cr;
		show: 'STARTING----->'.
	length
		timesRepeat: [ 
			[ 
			"Background updates"
			[ 
			0 to: 0 + length do: [ :i | 
				tt
					nextPutAll: '[' , i printString , ']';
					nextPutAll: '*';
					flush.
				Processor yield ] ] fork.	"Background updates"
			[ 
			500 to: 500 + length do: [ :i | 
				tt
					nextPutAll: '{' , i printString , '}';
					nextPutAll: '-';
					flush.
				Processor yield ] ] fork ] fork ].	"Display updates"
	length timesRepeat: [ [ [ length timesRepeat: [ World displayWorldSafely ] fork ] ] fork ].	"Foreground updates"
	1000 to: 1000 + length do: [ :i | 
		tt show: '---' , i printString , '---'.
		Processor yield ]! !

!ThreadSafeTranscript class methodsFor: 'declare' stamp: 'sd 3/27/2009 08:47'!
install

	self installThreadSafeAsTranscript! !

!ThreadSafeTranscript class methodsFor: 'declare' stamp: 'MarcusDenker 3/26/2010 16:56'!
installThreadSafeAsTranscript

	Smalltalk globals at: #Transcript put: (self new)

	"ThreadSafeTranscript open"! !

!ThreadSafeTranscript class methodsFor: 'world menu' stamp: 'MarcusDenker 3/24/2012 17:02'!
menuCommandOn: aBuilder 
	<worldMenu> 
	(aBuilder item: #'Transcript')
		action: [self open];
		parent: #Tools;
		help: 'Transcript';
		icon: self taskbarIcon.! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'NouryBouraqadi 10/18/2009 18:15'!
<< aString

	self show: aString! !

!ThreadSafeTranscript methodsFor: 'color' stamp: 'sd 5/18/2007 22:33'!
black
	"copied from Transcripter"

	Display depth = 1 ifTrue: [^ Bitmap with: 16rFFFFFFFF "Works without color support"].
	^ Color black! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'sd 5/19/2007 10:33'!
characterLimit

	^ 20000! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'lr 4/1/2009 10:23'!
clear
	"Clear all characters and redisplay the view"
	
	self changed: #clearText.
	accessSemaphore 
		critical: [ stream reset ]! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'lr 4/1/2009 10:25'!
close
	self flush.
	accessSemaphore critical: [ stream close ]
	! !

!ThreadSafeTranscript methodsFor: 'model protocol' stamp: 'yann.monclair 9/4/2009 16:26'!
codePaneMenu: aMenu shifted: shifted
	"Copied from TranscriptStream>>#codePaneMenu:shifted:"
	^ StringHolder basicNew codePaneMenu: aMenu shifted: shifted
! !

!ThreadSafeTranscript methodsFor: 'protected low level' stamp: 'lr 4/1/2009 10:23'!
contents 
	^ accessSemaphore critical: [ stream contents ]! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'sd 5/18/2007 15:05'!
cr 
	
	accessSemaphore
		critical: [stream cr].
! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'NouryBouraqadi 10/18/2009 18:12'!
crShow: anObject

	self cr; show: anObject! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'PavelKrivanek 7/12/2011 19:57'!
endEntry
  "Display all the characters since the last endEntry, and reset the
  stream "
  
  accessSemaphore critical: [
    self changed: #appendEntry.
    stream resetContents]! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'IgorStasenko 8/8/2011 15:02'!
ensureCr
	"do nothing, its here only for compatibility"! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'sd 5/18/2007 22:58'!
flush
	self endEntry
! !

!ThreadSafeTranscript methodsFor: 'ui building' stamp: 'NouryBouraqadi 10/18/2009 17:47'!
initialExtent
	
	^447@300! !

!ThreadSafeTranscript methodsFor: 'initialize' stamp: 'stephane.ducasse 4/1/2009 20:20'!
initialize
 
	super initialize.
	accessSemaphore := Mutex new.
	stream := String new writeStream.! !

!ThreadSafeTranscript methodsFor: 'printing' stamp: 'stephane.ducasse 3/31/2009 19:29'!
isSelfEvaluating

	self == Transcript ifTrue: [^true].
	^super isSelfEvaluating! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'sd 5/18/2007 15:04'!
nextPut: value
	
	accessSemaphore
		critical: [stream nextPut: value].
	^value! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'sd 5/18/2007 22:31'!
nextPutAll: value
	
	accessSemaphore
		critical: [stream nextPutAll: value].
	^value! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'sd 3/27/2009 08:43'!
pastEndPut: anObject
	"If the stream reaches its limit, just output the contents and reset."
	
	self endEntry.
	^ self nextPut: anObject! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'NouryBouraqadi 10/2/2009 15:59'!
print: anObject

	self nextPutAll: anObject asString! !

!ThreadSafeTranscript methodsFor: 'printing' stamp: 'stephane.ducasse 3/31/2009 19:33'!
printOn: aStream

	self == Transcript ifFalse: [^super printOn: aStream].
	aStream nextPutAll: 'Transcript'! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'stephane.ducasse 3/28/2009 21:32'!
reset
	! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'NouryBouraqadi 10/2/2009 15:59'!
show: anObject

	self print: anObject ; endEntry! !

!ThreadSafeTranscript methodsFor: 'protected low level' stamp: 'stephane.ducasse 3/28/2009 21:21'!
space

	accessSemaphore
		critical: [stream space].
! !

!ThreadSafeTranscript methodsFor: 'protected low level' stamp: 'sd 5/19/2007 10:36'!
tab
	
	accessSemaphore
		critical: [stream tab].
! !

!ThreadSafeTranscript methodsFor: 'ui building' stamp: 'lr 9/22/2009 11:40'!
title

	^ 'Transcript'! !

!ThreadSafeTranscript methodsFor: 'color' stamp: 'sd 5/18/2007 22:33'!
white
	"copied from Transcripter"

	Display depth = 1 ifTrue: [^ Bitmap with: 0 "Works without color support"].
	^ Color white! !

!ThreadSafeTranscript methodsFor: 'streaming' stamp: 'lr 4/1/2009 10:24'!
with: aBlock
	^ accessSemaphore critical: [ aBlock value: stream ]! !
